"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractAmqpBaseConsumer = void 0;
const core_1 = require("@message-queue-toolkit/core");
const AbstractAmqpService_1 = require("./AbstractAmqpService");
const amqpMessageReader_1 = require("./amqpMessageReader");
const ABORT_EARLY_EITHER = {
    error: 'abort',
};
class AbstractAmqpBaseConsumer extends AbstractAmqpService_1.AbstractAmqpService {
    transactionObservabilityManager;
    errorResolver;
    constructor(dependencies, options) {
        super(dependencies, options);
        this.transactionObservabilityManager = dependencies.transactionObservabilityManager;
        this.errorResolver = dependencies.consumerErrorResolver;
        if (!options.locatorConfig?.queueName && !options.creationConfig?.queueName) {
            throw new Error('queueName must be set in either locatorConfig or creationConfig');
        }
    }
    deserializeMessage(message) {
        if (message === null) {
            return ABORT_EARLY_EITHER;
        }
        const resolveMessageResult = this.resolveMessage(message);
        if ((0, core_1.isMessageError)(resolveMessageResult.error)) {
            this.handleError(resolveMessageResult.error);
            return ABORT_EARLY_EITHER;
        }
        // Empty content for whatever reason
        if (!resolveMessageResult.result) {
            return ABORT_EARLY_EITHER;
        }
        const resolveSchemaResult = this.resolveSchema(resolveMessageResult.result);
        if (resolveSchemaResult.error) {
            this.handleError(resolveSchemaResult.error);
            return ABORT_EARLY_EITHER;
        }
        const deserializationResult = (0, core_1.parseMessage)(resolveMessageResult.result, resolveSchemaResult.result, this.errorResolver);
        if ((0, core_1.isMessageError)(deserializationResult.error)) {
            this.handleError(deserializationResult.error);
            return ABORT_EARLY_EITHER;
        }
        // Empty content for whatever reason
        if (!deserializationResult.result) {
            return ABORT_EARLY_EITHER;
        }
        return {
            result: deserializationResult.result,
        };
    }
    async start() {
        await this.init();
        if (!this.channel) {
            throw new Error('Channel is not set');
        }
        await this.channel.consume(this.queueName, (message) => {
            if (message === null) {
                return;
            }
            const deserializedMessage = this.deserializeMessage(message);
            if (deserializedMessage.error === 'abort') {
                this.channel.nack(message, false, false);
                return;
            }
            // @ts-ignore
            const messageType = deserializedMessage.result[this.messageTypeField];
            const transactionSpanId = `queue_${this.queueName}:${
            // @ts-ignore
            // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
            deserializedMessage.result[this.messageTypeField]}`;
            this.transactionObservabilityManager?.start(transactionSpanId);
            this.processMessage(deserializedMessage.result, messageType)
                .then((result) => {
                if (result.error === 'retryLater') {
                    this.channel.nack(message, false, true);
                }
                if (result.result === 'success') {
                    this.channel.ack(message);
                }
            })
                .catch((err) => {
                // ToDo we need sanity check to stop trying at some point, perhaps some kind of Redis counter
                // If we fail due to unknown reason, let's retry
                this.channel.nack(message, false, true);
                this.handleError(err);
            })
                .finally(() => {
                this.transactionObservabilityManager?.stop(transactionSpanId);
            });
        });
    }
    resolveMessage(message) {
        return (0, amqpMessageReader_1.readAmqpMessage)(message, this.errorResolver);
    }
}
exports.AbstractAmqpBaseConsumer = AbstractAmqpBaseConsumer;
//# sourceMappingURL=AbstractAmqpBaseConsumer.js.map